/**
 * 用于存储小程序页面实例，更新页面值
 */
const instanceKey = Symbol("instance");
class InstanceStore {
    constructor() {
        // 事件储存
        this[instanceKey] = [];
    }
    /**
     * 新增监听页面/组件
     * @param {*} instance 页面/组件实例
     */
    push(instance) {
        this[instanceKey].push(instance);
    }
    /**
     * 移除监听
     * @param {*} instance 页面/组件实例
     */
    remove(instance) {
        this[instanceKey] = this[instanceKey].filter((ins) => ins !== instance);
    }
    /**
     * 更新实例数据
     * @param {*} stateKey 需要更新的键
     * @param {*} stateValue 值
     */
    update(stateKey, stateValue) {
        this[instanceKey].forEach((instance) => {
            instance.setData({
                [stateKey]: stateValue,
            });
        });
    }
}
const instanceStore = new InstanceStore();

// 状态代理
let store = {};
// 是否初始化
let isInit = false;
function createStore(state, callback) {
    // 如果已经初始化
    if (isInit) {
        throw new Error("Do not repeat initialization");
    }
    // 如果传入参数非对象
    if (Object.prototype.toString.call(state) !== "[object Object]") {
        throw new Error("state is not object");
    }
    const handler = {
        set(obj, prop, value) {
            obj[prop] = value;
            // 触发修改所有实例
            instanceStore.update(prop, value);
            // 回调函数
            if (typeof callback === "function") {
                callback(prop, value);
            }
            return true;
        },
    };
    // 初始化标记
    isInit = true;
    // store指向state
    store = state;
    return new Proxy(state, handler);
}
/**
 * 获取store
 */
function getStore() {
    return { ...store };
}

/**
 * 合并数据
 * @param {*} storeData store
 */
function syncData(storeData) {
    Object.keys(this.data).forEach((key) => {
        if (key in storeData) {
            this.setData({
                [key]: storeData[key],
            });
        }
    });
}
/**
 * 执行函数
 * @param {*} fn 函数
 * @param  {...any} args
 * @example
 * ```
 * useCallback.call(this, alert, arg)
 * ```
 */
function useCallback(fn, ...args) {
    if (typeof fn === "function") {
        fn.apply(this, args);
    }
}
/**
 * 页面合并数据响应式
 * @param options
 * @returns
 */
function componentReactive(options = {}) {
    const opt = { ...options };
    // 如果没有设置lifetimes
    if (opt.lifetimes === undefined) {
        opt.lifetimes = {};
    }
    // 加载完成
    opt.lifetimes.attached = function () {
        // 同步state数据
        syncData.call(this, getStore());
        useCallback.call(this, options.lifetimes?.attached);
        // 监听当前页面
        instanceStore.push(this);
    };
    // 销毁
    opt.lifetimes.detached = function () {
        useCallback.call(this, options.lifetimes?.detached);
        // 监听当前页面
        instanceStore.remove(this);
    };
    return opt;
}
/**
 * 页面数据合并
 * @param options
 */
function pageReactive(options = {}) {
    const opt = { ...options };
    /**
     * 页面加载时候同步数据到Data,并监听当前页面数据变化
     * @param this
     * @param res
     */
    opt.onLoad = function (res) {
        // 同步state数据
        syncData.call(this, getStore());
        useCallback.call(this, options?.onLoad, res);
        // 监听当前页面
        instanceStore.push(this);
    };
    /**
     * 页面卸载
     */
    opt.onUnload = function () {
        useCallback.call(this, options?.onUnload);
        // 移除监听
        instanceStore.remove(this);
    };
    return opt;
}

export { componentReactive, createStore, getStore, pageReactive };
