import { getStore } from "../store/index";
import { instanceStore } from "../observe/index";

type WXinstance = WechatMiniprogram.Page.Instance<
   Record<string, unknown>,
   Record<string, unknown>
>;

/**
 * 合并数据
 * @param {*} storeData store
 */
function syncData(this: WXinstance, storeData: Record<string, unknown>) {
   Object.keys(this.data).forEach((key) => {
      if (key in storeData) {
         this.setData({
            [key]: storeData[key],
         });
      }
   });
}

/**
 * 执行函数
 * @param {*} fn 函数
 * @param  {...any} args
 * @example
 * ```
 * useCallback.call(this, alert, arg)
 * ```
 */
function useCallback(this: unknown, fn: unknown, ...args: unknown[]) {
   if (typeof fn === "function") {
      fn.apply(this, args);
   }
}

/**
 * 合并
 * @param {*} options
 */
type WechatPageOptions = WechatMiniprogram.Page.Options<
   WechatMiniprogram.Page.DataOption,
   WechatMiniprogram.Page.CustomOption
>;
type WechatComponentOptions = WechatMiniprogram.Component.Options<
   WechatMiniprogram.Component.DataOption,
   WechatMiniprogram.Component.PropertyOption,
   WechatMiniprogram.Component.MethodOption
>;

/**
 * 页面合并数据响应式
 * @param options
 * @returns
 */
export function componentReactive(
   options: WechatComponentOptions = {},
): WechatComponentOptions {
   const opt = { ...options };

   // 如果没有设置lifetimes
   if (opt.lifetimes === undefined) {
      opt.lifetimes = {};
   }

   // 加载完成
   opt.lifetimes.attached = function (this: WXinstance) {
      // 同步state数据
      syncData.call(this, getStore());

      useCallback.call(this, options.lifetimes?.attached);

      // 监听当前页面
      instanceStore.push(this);
   };

   // 销毁
   opt.lifetimes.detached = function (this: WXinstance) {
      useCallback.call(this, options.lifetimes?.detached);

      // 监听当前页面
      instanceStore.remove(this);
   };

   return opt;
}

/**
 * 页面数据合并
 * @param options
 */
export function pageReactive(
   options: WechatPageOptions = {},
): WechatPageOptions {
   const opt = { ...options };

   /**
    * 页面加载时候同步数据到Data,并监听当前页面数据变化
    * @param this
    * @param res
    */
   opt.onLoad = function (this: WXinstance, res) {
      // 同步state数据
      syncData.call(this, getStore());

      useCallback.call(this, options?.onLoad, res);

      // 监听当前页面
      instanceStore.push(this);
   };

   /**
    * 页面卸载
    */
   opt.onUnload = function (this: WXinstance) {
      useCallback.call(this, options?.onUnload);

      // 移除监听
      instanceStore.remove(this);
   };

   return opt;
}
