import { instanceStore } from "../observe/index";

type StroreType = Record<string, unknown>;

// 状态代理
let store: StroreType = {};
// 是否初始化
let isInit = false;

/**
 * 创建Store
 * @param {*} state 数据
 */
interface StroreCallback {
   /**
    * store变更后的回调
    * @param prop 改动的属性
    * @param value 改动的值
    * @returns
    */
   (prop: string, value: unknown): void;
}
export function createStore(
   state: StroreType,
   callback?: StroreCallback,
): StroreType | Error {
   // 如果已经初始化
   if (isInit) {
      throw new Error("Do not repeat initialization");
   }

   // 如果传入参数非对象
   if (Object.prototype.toString.call(state) !== "[object Object]") {
      throw new Error("state is not object");
   }

   const handler = {
      set(obj: StroreType, prop: string, value: unknown) {
         obj[prop as keyof StroreType] = value;

         // 触发修改所有实例
         instanceStore.update(prop, value);

         // 回调函数
         if (typeof callback === "function") {
            callback(prop, value);
         }

         return true;
      },
   };

   // 初始化标记
   isInit = true;

   // store指向state
   store = state;

   return new Proxy(state, handler);
}

/**
 * 获取store
 */
export function getStore(): StroreType {
   return { ...store };
}
